@@ -1,7 +1,7 @@
-#![doc(html_root_url = "https://docs.rs/exmex/0.12.0")]
+#![doc(html_root_url = "https://docs.rs/exmex/0.14.0")]
 //! Exmex is an extendable mathematical expression parser and evaluator. Ease of use, flexibility, and efficient evaluations are its main design goals.
-//! Exmex can parse mathematical expressions possibly containing variables and operators. On the one hand, it comes with a list of default operators 
-//! for floating point values. For differentiable default operators, Exmex can compute partial derivatives. On the other hand, users can define their 
+//! Exmex can parse mathematical expressions possibly containing variables and operators. On the one hand, it comes with a list of default operators
+//! for floating point values. For differentiable default operators, Exmex can compute partial derivatives. On the other hand, users can define their
 //! own operators and work with different data types such as float, integer, bool, or other types that implement `Clone`, `FromStr`, and `Debug`.
 //!
 //! The following snippet shows how to evaluate a string.
@@ -63,69 +63,37 @@
 //! The value returned by [`parse`](parse) is an instance of the struct [`FlatEx`](FlatEx)
 //! that implements the [`Express`](Express) trait. Moreover, [`FlatEx`](FlatEx) and
 //! [`Express`](Express) are the only items made accessible by the wildcard import from
-//! [`prelude`](prelude).
+//! [`prelude`](prelude) if the feature `partial` is not used.
 //!
-//! ## Partial Derivatives
-//!
-//! For default operators, expressions can be transformed into their partial derivatives
-//! again represented by expressions. To this end, there exists the method [`partial`](Express::partial).
-//! ```rust
-//! # use std::error::Error;
-//! # fn main() -> Result<(), Box<dyn Error>> {
-//! #
-//! use exmex::prelude::*;
-//! let expr = exmex::parse::<f64>("x^2 + y^2")?;
-//! let dexpr_dx = expr.clone().partial(0)?;
-//! let dexpr_dy = expr.partial(1)?;
-//! assert!((dexpr_dx.eval(&[3.0, 2.0])? - 6.0).abs() < 1e-12);
-//! assert!((dexpr_dy.eval(&[3.0, 2.0])? - 4.0).abs() < 1e-12);
-//! #
-//! #     Ok(())
-//! # }
+//! ## Features
+//! Exmex comes with three features that can be activated in the `Cargo.toml` via
+//! ```text
+//! [dependencies]
+//! exmex = { ..., features = ["partial", "serde", "value"] }
 //! ```
+//! 
+//! `partial` allows the computation of partal derivatives, `serde` enables serialization and 
+//! deserialization, and `value` makes a more general value type accessible.
+//! 
+//! ### Partial Derivatives
 //!
-//! ## Owned Expression
-//! You cannot return all expression types from a function without a lifetime parameter.
-//! For instance, expressions that are instances of [`FlatEx`](FlatEx) keep `&str`s instead of
-//! `String`s of variable or operator names to make faster parsing possible.
-//! ```rust
-//! # use std::error::Error;
-//! # fn main() -> Result<(), Box<dyn Error>> {
-//! #
-//! use exmex::prelude::*;
-//! use exmex::ExResult;
-//! fn create_expr<'a>() -> ExResult<FlatEx::<'a, f64>> {
-//! //              |                          |
-//! //              lifetime parameter necessary
+//! Expressions with floating point data types can be transformed into their 
+//! partial derivatives again represented by expressions after activating the feature `partial`.
+//! See the [readme](https://github.com/bertiqwerty/exmex#partial-differentiation) for examples.
+//! 
+//! ### Serialization and Deserialization
 //!
-//!     let to_be_parsed = "log(z) + 2* (-z^2 + sin(4*y))";
-//!     exmex::parse::<f64>(to_be_parsed)
-//! }
-//! let expr = create_expr()?;
-//! assert!((expr.eval(&[3.7, 2.5])? - 14.992794866624788 as f64).abs() < 1e-12);
-//! #
-//! #     Ok(())
-//! # }
-//! ```
-//! If you are willing to pay the price of roughly doubled parsing times, you can
-//! obtain an expression that is an instance of [`OwnedFlatEx`](OwnedFlatEx) and owns
-//! its strings. Evaluation times should be comparable. However, a lifetime parameter is
-//! not needed anymore as shown in the following.
-//! ```rust
-//! # use std::error::Error;
-//! # fn main() -> Result<(), Box<dyn Error>> {
-//! #
-//! use exmex::{ExResult, Express, OwnedFlatEx};
-//! fn create_expr() -> ExResult<OwnedFlatEx::<f64>> {
-//!     let to_be_parsed = "log(z) + 2* (-z^2 + sin(4*y))";
-//!     OwnedFlatEx::<f64>::from_str(to_be_parsed)
-//! }
-//! let expr_owned = create_expr()?;
-//! assert!((expr_owned.eval(&[3.7, 2.5])? - 14.992794866624788 as f64).abs() < 1e-12);
-//! #
-//! #     Ok(())
-//! # }
-//! ```
+//! To use [`serde`](https://serde.rs/) you can activate the feature `serde`.
+//! The implementation un-parses and re-parses the whole expression.
+//! [`Deserialize`](https://docs.serde.rs/serde/de/trait.Deserialize.html) and
+//! [`Serialize`](https://docs.serde.rs/serde/de/trait.Serialize.html) are implemented for
+//! [`FlatEx`](FlatEx).
+//!
+//! ### A more General Value Type
+//!
+//! To use different data types within an expression, one can activate the feature `value` and
+//! use the more general type `Val`. The additional flexibility comes with higher parsing
+//! and evaluation run times, see the [benchmarks](https://github.com/bertiqwerty/exmex#benchmarks-v0130).
 //!
 //! ## Extendability
 //!
@@ -221,17 +189,20 @@
 //! ### Custom Data Types of Numbers
 //!
 //! You can use any type that implements [`Clone`](Clone),
-//! [`FromStr`](std::str::FromStr), and [`Debug`](std::fmt::Debug). In case the representation of your data type in the
-//! string does not match the number regex `r"^(\.?[0-9]+(\.[0-9]+)?)"`, you have to pass a
-//! suitable regex and use the function
-//! [`from_pattern`](Express::from_pattern) instead of [`parse`](crate::parse) or
-//! [`from_str`](Express::from_str). Here is an example for `bool`.
+//! [`FromStr`](std::str::FromStr), and [`Debug`](std::fmt::Debug). In case the representation of your data type's literals
+//! in the string does not match the number regex `r"^(\.?[0-9]+(\.[0-9]+)?)"`, you have to create a suitable matcher
+//! type that implements [`MatchLiteral`](MatchLiteral). Given a suitable regex pattern, you can utilize the macro
+//! [`literal_matcher_from_pattern`](literal_matcher_from_pattern).
+//! Here is an example for `bool`.
 //! ```rust
 //! # use std::error::Error;
 //! # fn main() -> Result<(), Box<dyn Error>> {
 //! #
 //! use exmex::prelude::*;
-//! use exmex::{BinOp, MakeOperators, Operator, ops_factory};
+//! use exmex::{
+//!     BinOp, MakeOperators, MatchLiteral, Operator,
+//!     literal_matcher_from_pattern, ops_factory
+//! };
 //! ops_factory!(
 //!     BooleanOpsFactory,
 //!     bool,
@@ -253,16 +224,17 @@
 //!     ),
 //!     Operator::make_unary("!", |a| !a)
 //! );
+//! literal_matcher_from_pattern!(BooleanMatcher, "^(true|false)");
 //! let to_be_parsed = "!(true && false) || (!false || (true && false))";
-//! let expr = FlatEx::<_, BooleanOpsFactory>::from_pattern(to_be_parsed, "^(true|false)")?;
+//! type FlatExBool = FlatEx::<bool, BooleanOpsFactory, BooleanMatcher>;
+//! let expr = FlatExBool::from_str(to_be_parsed)?;
 //! assert_eq!(expr.eval(&[])?, true);
 //! #
 //! #     Ok(())
 //! # }
 //! ```
-//! You can also pre-compile the regex and use [`Express::from_regex`](Express::from_regex).
 //! Two examples of exmex with non-trivial data types are:
-//! * Numbers can be operators and operators can operate on operators, see, e.g., 
+//! * Numbers can be operators and operators can operate on operators, see, e.g.,
 //! also a blog post on [ninety.de](https://www.ninety.de/log/index.php/en/2021/11/11/parsing-operators-in-rust/).
 //! * The value type implemented as part of the feature `value` allows expressions containing integers, floats, and bools.
 //! Therewith, Pythonesque expressions of the form `"x if a > b else y"` are possible.
@@ -285,10 +257,9 @@
 //!
 //! ## Display
 //!
-//! Instances of [`FlatEx`](FlatEx) and [`OwnedFlatEx`](OwnedFlatEx) can be displayed as string. Note that this
-//! [`unparse`](Express::unparse)d string does not necessarily coincide with the original
-//! string, since, e.g., curly brackets are added, expressions are compiled, and constants are
-//! replaced by their numeric values during parsing.
+//! Expression can be displayed as string. This
+//! [`unparse`](Express::unparse)d string coincides with the original
+//! string.
 //!
 //! ```rust
 //! # use std::error::Error;
@@ -296,37 +267,16 @@
 //! #
 //! use exmex::prelude::*;
 //! let expr = exmex::parse::<f64>("-sin(z)/cos(mother_of_names) + 2^7 + E")?;
-//! assert_eq!(format!("{}", expr), "-(sin({z}))/cos({mother_of_names})+130.71828182845906");
+//! assert_eq!(format!("{}", expr), "-sin(z)/cos(mother_of_names) + 2^7 + E");
 //! #
 //! #     Ok(())
 //! # }
 //! ```
 //!
-//! ## Features
-//! Exmex comes with two features that can be activated in the `Cargo.toml` via
-//! ```text
-//! [dependencies]
-//! exmex = { ..., features = ["serde", "value"] }
-//! ```
-//! `serde` enables serialization and deserialization and `value` a more general value type.
-//! ### Serialization and Deserialization
-//!
-//! To use [`serde`](https://serde.rs/) you can activate the feature `serde`.
-//! The implementation un-parses and re-parses the whole expression.
-//! [`Deserialize`](https://docs.serde.rs/serde/de/trait.Deserialize.html) and
-//! [`Serialize`](https://docs.serde.rs/serde/de/trait.Serialize.html) are implemented for
-//! both, [`FlatEx`](FlatEx) and [`OwnedFlatEx`](OwnedFlatEx).
-//! 
-//! ### A more General Value Type
-//! 
-//! To use different data types within an expression, one can activate the feature `value`. The additional
-//! flexibility comes with higher parsing (factor 1.1-1.4) and evaluation run times (factor 2-3).
-//!
 
 use std::{fmt::Debug, str::FromStr};
 
 use data_type::DataType;
-use expression::deep::DeepEx;
 use num::Float;
 mod definitions;
 mod expression;
@@ -339,8 +289,8 @@ mod util;
 
 pub use {
     expression::{
-        flat::{FlatEx, OwnedFlatEx},
-        Express,
+        flat::FlatEx,
+        Express, MatchLiteral, NumberMatcher,
     },
     operators::{BinOp, FloatOpsFactory, MakeOperators, Operator},
     result::{ExError, ExResult},
@@ -350,13 +300,26 @@ pub use {
 mod value;
 #[cfg(feature = "value")]
 pub use value::{
-    parse_val, parse_val_owned, FlatExVal, OwnedFlatExVal, Val, ValOpsFactory,
+    parse_val, FlatExVal, Val, ValMatcher,
+    ValOpsFactory,
 };
+#[cfg(feature = "partial")]
+mod partial;
+#[cfg(feature = "partial")]
+pub use partial::Differentiate;
 
-/// To use the expression trait [`Express`](Express) and its implementation [`FlatEx`](FlatEx)
-/// one can `use exmex::prelude::*;`.
+/// Exmex' prelude can be imported via can `use exmex::prelude::*;`.
+/// 
+/// The prelude contains
+/// * expression trait [`Express`](Express), 
+/// * its implementation [`FlatEx`](FlatEx),
+/// * and the partial differentiation of [`FlatEx`](FlatEx), if the feature `partial` is active. 
+/// 
 pub mod prelude {
-    pub use super::expression::{flat::FlatEx, Express};
+    pub use crate::expression::{flat::FlatEx, Express};
+    pub use std::str::FromStr;
+    #[cfg(feature = "partial")]
+    pub use crate::Differentiate;
 }
 
 /// Parses a string, evaluates the expression, and returns the resulting number.
@@ -370,14 +333,13 @@ pub fn eval_str<T: Float + DataType>(text: &str) -> ExResult<T>
 where
     <T as FromStr>::Err: Debug,
 {
-    let deepex = DeepEx::<T>::from_str_float(text)?;
-    if deepex.n_vars() > 0 {
+    let flatex = FlatEx::<T>::from_str_wo_compile(text)?;
+    if !flatex.var_names().is_empty() {
         return Err(ExError {
             msg: format!("input string contains variables, '{}' ", text),
         });
     }
-
-    deepex.eval(&[])
+    flatex.eval(&[])
 }
 
 /// Parses a string and returns the expression that can be evaluated.
