@@ -1,22 +1,23 @@
-use crate::{ExError, ExResult, definitions::N_UNARYOPS_OF_DEEPEX_ON_STACK, format_exerr};
+use crate::{definitions::N_UNARYOPS_OF_DEEPEX_ON_STACK, format_exerr, ExError, ExResult};
 use num::Float;
 use smallvec::{smallvec, SmallVec};
 use std::{fmt::Debug, marker::PhantomData};
 
 enum OperatorType {
     Bin,
-    Unary
+    Unary,
 }
 
 fn make_op_not_available_error(repr: &str, op_type: OperatorType) -> ExError {
     let op_type_str = match op_type {
         OperatorType::Bin => "binary",
-        OperatorType::Unary => "unary"
+        OperatorType::Unary => "unary",
     };
-    format_exerr!("{} operator {} not available", op_type_str, repr)    
+    format_exerr!("{} operator '{}' not available", op_type_str, repr)
 }
 
-/// Operators can be custom-defined by the library-user in terms of this struct.
+/// Operators can be unary such as `sin`, binary such as `*`, unary and binary such as `-`, 
+/// or constants such as `π`. To use custom operators, see also the macro [`ops_factory`](ops_factory).
 #[derive(Clone, Eq, PartialEq, Ord, PartialOrd, Debug)]
 pub struct Operator<'a, T: Clone> {
     /// Representation of the operator in the string to be parsed, e.g., `-` or `sin`.
@@ -30,8 +31,14 @@ pub struct Operator<'a, T: Clone> {
     constant: Option<T>,
 }
 
-fn unwrap_operator<'a, O>(wrapped_op: &'a Option<O>, repr: &str, op_type: OperatorType) -> ExResult<&'a O> {
-    wrapped_op.as_ref().ok_or_else(||make_op_not_available_error(repr, op_type))
+fn unwrap_operator<'a, O>(
+    wrapped_op: &'a Option<O>,
+    repr: &str,
+    op_type: OperatorType,
+) -> ExResult<&'a O> {
+    wrapped_op
+        .as_ref()
+        .ok_or_else(|| make_op_not_available_error(repr, op_type))
 }
 
 impl<'a, T: Clone> Operator<'a, T> {
@@ -91,7 +98,11 @@ impl<'a, T: Clone> Operator<'a, T> {
         Ok(op.clone())
     }
     pub fn unary(&self) -> ExResult<fn(T) -> T> {
-        Ok(*unwrap_operator(&self.unary_op, self.repr, OperatorType::Unary)?)
+        Ok(*unwrap_operator(
+            &self.unary_op,
+            self.repr,
+            OperatorType::Unary,
+        )?)
     }
     pub fn repr(&self) -> &'a str {
         self.repr
@@ -115,8 +126,11 @@ pub struct UnaryOp<T> {
     funcs_to_be_composed: VecOfUnaryFuncs<T>,
 }
 
-impl<T> UnaryOp<T> {
-    /// Applies unary operators one after the other starting with the last.
+impl<T> UnaryOp<T>
+where
+    T: Clone,
+{
+    /// Applies unary operators one after the other starting with the one with the highest index.
     /// # Arguments
     ///
     /// * `x` - number the unary operators are applied to
@@ -130,12 +144,27 @@ impl<T> UnaryOp<T> {
         result
     }
 
-    pub fn append_front(&mut self, other: &mut UnaryOp<T>) {
-        self.funcs_to_be_composed = other
-            .funcs_to_be_composed
-            .iter()
-            .chain(self.funcs_to_be_composed.iter())
-            .copied()
+    /// Composes `self` with another unary operator.
+    /// The other unary operator will be applied after self.
+    pub fn append_after(&mut self, other: &UnaryOp<T>) {
+        self.append_after_iter(other.funcs_to_be_composed.iter().copied());
+    }
+
+    
+    /// Removes the operator that will be applied latest, i.e., the first element of the array.
+    pub fn remove_latest(&mut self) {
+        self.funcs_to_be_composed.remove(0);
+    }
+
+    /// Appends an iterator of unary functions to the beginning of the array of unary functions of `self`.
+    /// Accordingly, the newly added unary functions will be applied after all other unary functions in the
+    /// list, i.e., as latest.
+    pub fn append_after_iter<I>(&mut self, other_iter: I)
+    where
+        I: Iterator<Item = fn(T) -> T>,
+    {
+        self.funcs_to_be_composed = other_iter
+            .chain(self.funcs_to_be_composed.iter().copied())
             .collect::<SmallVec<_>>();
     }
 
@@ -155,11 +184,30 @@ impl<T> UnaryOp<T> {
         }
     }
 
+    pub fn from_iter<I>(iter: I) -> Self
+    where
+        I: Iterator<Item = fn(T) -> T>,
+    {
+        Self {
+            funcs_to_be_composed: iter.collect(),
+        }
+    }
+    
+    pub fn funcs_to_be_composed(&self) -> &VecOfUnaryFuncs<T> {
+        &self.funcs_to_be_composed
+    }
+
     pub fn clear(&mut self) {
         self.funcs_to_be_composed.clear();
     }
 }
 
+impl<T: Clone> Default for UnaryOp<T> {
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
 /// A binary operator that consists of a function pointer, a priority, and a commutativity-flag.
 #[derive(Clone, Eq, PartialEq, Ord, PartialOrd, Debug)]
 pub struct BinOp<T: Clone> {
@@ -169,7 +217,7 @@ pub struct BinOp<T: Clone> {
     /// higher number will be executed first. For instance, in a sane world `*`
     /// has a higher priority than `+`. However, in Exmex land you could also define
     /// this differently.
-    pub prio: i32,
+    pub prio: i64,
     /// True if this is a commutative operator such as `*` or `+`, false if not such as `-`, `/`, or `^`.
     /// Commutativity is used to compile sub-expressions of numbers correctly.
     pub is_commutative: bool,
@@ -208,7 +256,7 @@ pub trait MakeOperators<T: Clone>: Clone {
 }
 
 /// Factory of default operators for floating point values.
-/// 
+///
 /// |representation|description|
 /// |--------------|-----------|
 /// |`^`| power |
@@ -239,8 +287,8 @@ pub trait MakeOperators<T: Clone>: Clone {
 /// |`PI`| constant π  |
 /// |`π`| second representations of constant π  |
 /// |`E`| Euler's number |
-/// 
-/// 
+///
+///
 #[derive(Clone, Eq, PartialEq, Ord, PartialOrd, Debug)]
 pub struct FloatOpsFactory<T: Float> {
     dummy: PhantomData<T>,
