@@ -1,5 +1,5 @@
 use crate::data_type::DataType;
-use crate::definitions::N_NODES_ON_STACK;
+use crate::definitions::{N_NODES_ON_STACK, N_VARS_ON_STACK};
 use crate::format_exerr;
 use crate::{operators::Operator, ExError, ExResult};
 use lazy_static::lazy_static;
@@ -21,6 +21,57 @@ pub enum ParsedToken<'a, T: DataType> {
     Var(&'a str),
 }
 
+/// Returns the index of the variable in the slice. Panics if not available!
+pub fn find_var_index<'a>(name: &str, parsed_vars: &[&'a str]) -> usize {
+    let idx = parsed_vars.iter().enumerate().find(|(_, n)| **n == name);
+    match idx {
+        Some((i, _)) => i,
+        None => {
+            panic!("This is probably a bug. I don't know variable {}", name)
+        }
+    }
+}
+
+/// Disambiguates operators based on predecessor token.
+pub fn is_operator_binary<'a, T: DataType>(
+    op: &Operator<'a, T>,
+    parsed_token_on_the_left: &ParsedToken<'a, T>,
+) -> ExResult<bool> {
+    if op.has_bin() && !op.has_unary() {
+        match parsed_token_on_the_left {
+            ParsedToken::Op(op_) => {
+                Err(format_exerr!("a binary operator cannot be on the right another operator, {:?} next to {:?}", op, op_))
+            },
+            _ => Ok(true)
+        }
+    } else if op.has_bin() && op.has_unary() {
+        Ok(match parsed_token_on_the_left {
+            ParsedToken::Num(_) | ParsedToken::Var(_) => true,
+            ParsedToken::Paren(p) => *p == Paren::Close,
+            ParsedToken::Op(_) => false,
+        })
+    } else {
+        Ok(false)
+    }
+}
+
+/// Returns variable names in sorted order.
+pub fn find_parsed_vars<'a, T: DataType>(
+    parsed_tokens: &[ParsedToken<'a, T>],
+) -> SmallVec<[&'a str; N_VARS_ON_STACK]> {
+    let mut found_vars = SmallVec::<[&str; N_VARS_ON_STACK]>::new();
+    for pt in parsed_tokens {
+        match pt {
+            ParsedToken::Var(name) if !found_vars.contains(name) => {
+                found_vars.push(*name);
+            }
+            _ => (),
+        }
+    }
+    found_vars.sort_unstable();
+    found_vars
+}
+
 pub fn is_numeric_text(text: &str) -> Option<&str> {
     let mut n_dots = 0;
     let n_num_chars = text
@@ -40,14 +91,6 @@ pub fn is_numeric_text(text: &str) -> Option<&str> {
     }
 }
 
-pub fn is_numeric_regex<'a>(re: &Regex, text: &'a str) -> Option<&'a str> {
-    let maybe_num = re.find(text);
-    match maybe_num {
-        Some(m) => Some(m.as_str()),
-        None => None,
-    }
-}
-
 fn next_char_boundary(text: &str, start_idx: usize) -> usize {
     (1..text.len())
         .find(|idx| text.is_char_boundary(start_idx + idx))
@@ -368,8 +411,7 @@ fn test_preconditions() {
             match err {
                 Ok(_) => {
                     println!("expected an error that should contain '{}'", msg_part);
-
-                    assert!(false)
+                    unreachable!();
                 }
                 Err(e) => {
                     println!("msg '{}' should contain '{}'", e.msg, msg_part);
@@ -380,6 +422,7 @@ fn test_preconditions() {
 
         let ops = FloatOpsFactory::<f32>::make();
         let elts = tokenize_and_analyze(text, &ops, is_numeric_text);
+        println!("{:?}", elts);
         match elts {
             Err(e) => check_err_msg::<Vec<ParsedToken<f32>>>(Err(e), msg_part),
             Ok(elts) => {
