@@ -10,7 +10,7 @@ pub struct ExError {
     pub msg: String,
 }
 impl ExError {
-    pub fn from_str(msg: &str) -> ExError {
+    pub fn new(msg: &str) -> ExError {
         ExError {
             msg: msg.to_string(),
         }
@@ -26,7 +26,7 @@ impl Error for ExError {}
 /// Exmex' result type with [`ExError`](ExError) as error type.
 pub type ExResult<U> = Result<U, ExError>;
 
-/// Creates an [`ExError`](ExError) with formatted message.
+/// Creates an [`ExError`](ExError) with a formatted message.
 /// ```rust
 /// # use std::error::Error;
 /// use exmex::{format_exerr, ExError};
