@@ -1,26 +1,12 @@
 use std::{cmp::Ordering, fmt::Debug, marker::PhantomData, str::FromStr};
 
-use lazy_static::lazy_static;
 use num::{Float, PrimInt, Signed};
-use regex::Regex;
 
 use crate::{
-    data_type::DataType, format_exerr, BinOp, ExError, ExResult, Express, FlatEx, MakeOperators,
-    Operator, OwnedFlatEx,
+    data_type::DataType, expression::MatchLiteral, format_exerr, literal_matcher_from_pattern,
+    BinOp, ExError, ExResult, FlatEx, MakeOperators, Operator,
 };
 
-lazy_static! {
-    static ref RE_VAR_NAME_EXACT: Regex =
-        Regex::new(r"^([0-9]+(\.[0-9]+)?|true|false|\[\s*(\-?.?[0-9]+(\.[0-9]+)?|true|false)(\s*,\s*-?\.?[0-9]+(\.[0-9]+)?|true|false)*\s*\])").unwrap();
-}
-
-/// *`feature = "value"`* - Alias for [`FlatEx`](FlatEx) with [`Val`](Val) as data type and [`ValOpsFactory`](ValOpsFactory)
-/// as operator factory.
-pub type FlatExVal<'a, I, F> = FlatEx<'a, Val<I, F>, ValOpsFactory<I, F>>;
-/// *`feature = "value"`* - Alias for [`OwnedFlatEx`](OwnedFlatEx) with [`Val`](Val) as data type and [`ValOpsFactory`](ValOpsFactory)
-/// as operator factory.
-pub type OwnedFlatExVal<'a, I, F> = OwnedFlatEx<Val<I, F>, ValOpsFactory<I, F>>;
-
 macro_rules! to_type {
     ($name:ident, $T:ty, $variant:ident) => {
         pub fn $name(self) -> ExResult<$T> {
@@ -38,11 +24,9 @@ macro_rules! to_type {
 
 /// *`feature = "value"`* -
 /// The value type [`Val`](Val) can contain an integer, float, bool, none, or error.
-/// To use the value type, there are the separate parse functions [`parse_val`](`parse_val`) and 
-/// [`parse_val_owned`](`parse_val_owned`) that wrap [`Express::from_regex`](Express::from_regex)
-/// and use the corresponding operator factory [`ValOpsFactory`](ValOpsFactory). In the following example,
-/// the ternary Python-style `a if condition else b` is used. This is equivalent to `if condition {a} else {b}` in Rust
-/// or `condition ? a : b` in C.
+/// To use the value type, there are the is a parse function [`parse_val`](`parse_val`).
+/// In the following example, the ternary Python-style `a if condition else b` is used.
+/// This is equivalent to `if condition {a} else {b}` in Rust or `condition ? a : b` in C.
 /// ```rust
 /// # use std::error::Error;
 /// # fn main() -> Result<(), Box<dyn Error>> {
@@ -140,7 +124,7 @@ where
     type Err = ExError;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
-        let res = Ok(if s.contains(".") {
+        let res = Ok(if s.contains('.') {
             Val::Float(s.parse::<F>().map_err(map_parse_err)?)
         } else if s == "false" || s == "true" {
             Val::Bool(s.parse::<bool>().map_err(map_parse_err)?)
@@ -173,10 +157,6 @@ where
             _ => false,
         }
     }
-
-    fn ne(&self, other: &Val<I, F>) -> bool {
-        !(self == other)
-    }
 }
 
 impl<I, F> PartialOrd<Val<I, F>> for Val<I, F>
@@ -215,7 +195,7 @@ where
                 y
             )),
         },
-        _ => Val::Error(ExError::from_str("cannot compute power of")),
+        _ => Val::Error(ExError::new("cannot compute power of")),
     }
 }
 
@@ -239,7 +219,7 @@ macro_rules! base_arith {
                 },
                 (Val::Float(x), Val::Int(y)) => Val::Float(x.$name(F::from(y).unwrap())),
                 (Val::Int(x), Val::Float(y)) => Val::Float(F::from(x).unwrap().$name(y)),
-                _ => Val::Error(ExError::from_str(
+                _ => Val::Error(ExError::new(
                     format!("can only apply {} to ints or floats", stringify!($name)).as_str(),
                 )),
             }
@@ -272,7 +252,7 @@ macro_rules! single_type_arith {
 }
 
 single_type_arith!(rem, Int, |a, b| if b == I::zero() {
-    Val::Error(ExError::from_str("% by zero"))
+    Val::Error(ExError::new("% by zero"))
 } else {
     Val::Int(a % b)
 });
@@ -376,12 +356,13 @@ unary_op!(
                 Some(x) => x,
                 None => return Val::Error(format_exerr!("cannot compute factorial of {:?}", a)),
             };
-            let res = (1usize..(a_usize_unpacked + 1usize))
-                .map(|i: usize| I::from(i))
-                .fold(Some(I::one()), |a, b| match (a, b) {
-                    (Some(a_), Some(b_)) => Some(a_ * b_),
-                    _ => None,
-                });
+            let res =
+                (1usize..(a_usize_unpacked + 1usize))
+                    .map(I::from)
+                    .fold(Some(I::one()), |a, b| match (a, b) {
+                        (Some(a_), Some(b_)) => Some(a_ * b_),
+                        _ => None,
+                    });
             match res {
                 Some(i) => Val::Int(i),
                 None => Val::Error(format_exerr!("cannot compute factorial of {:?}", a)),
@@ -463,7 +444,7 @@ where
             Operator::make_bin(
                 "^",
                 BinOp {
-                    apply: |a, b| pow(a, b),
+                    apply: pow,
                     prio: 6,
                     is_commutative: false,
                 },
@@ -471,7 +452,7 @@ where
             Operator::make_bin(
                 "+",
                 BinOp {
-                    apply: |a, b| add(a, b),
+                    apply: add,
                     prio: 3,
                     is_commutative: true,
                 },
@@ -479,16 +460,16 @@ where
             Operator::make_bin_unary(
                 "-",
                 BinOp {
-                    apply: |a, b| sub(a, b),
+                    apply: sub,
                     prio: 3,
                     is_commutative: false,
                 },
-                |a| minus(a),
+                minus,
             ),
             Operator::make_bin(
                 "*",
                 BinOp {
-                    apply: |a, b| mul(a, b),
+                    apply: mul,
                     prio: 4,
                     is_commutative: true,
                 },
@@ -498,7 +479,7 @@ where
                 BinOp {
                     apply: |a, b| match b {
                         Val::Int(x) if x == I::zero() => {
-                            Val::Error(ExError::from_str("int division by zero"))
+                            Val::Error(ExError::new("int division by zero"))
                         }
                         _ => div(a, b),
                     },
@@ -509,7 +490,7 @@ where
             Operator::make_bin(
                 "%",
                 BinOp {
-                    apply: |a, b| rem(a, b),
+                    apply: rem,
                     prio: 5,
                     is_commutative: false,
                 },
@@ -517,7 +498,7 @@ where
             Operator::make_bin(
                 "|",
                 BinOp {
-                    apply: |a, b| bitwise_or(a, b),
+                    apply: bitwise_or,
                     prio: 2,
                     is_commutative: true,
                 },
@@ -525,7 +506,7 @@ where
             Operator::make_bin(
                 "&",
                 BinOp {
-                    apply: |a, b| bitwise_and(a, b),
+                    apply: bitwise_and,
                     prio: 2,
                     is_commutative: true,
                 },
@@ -533,7 +514,7 @@ where
             Operator::make_bin(
                 "XOR",
                 BinOp {
-                    apply: |a, b| bitwise_xor(a, b),
+                    apply: bitwise_xor,
                     prio: 2,
                     is_commutative: true,
                 },
@@ -541,7 +522,7 @@ where
             Operator::make_bin(
                 ">>",
                 BinOp {
-                    apply: |a, b| right_shift(a, b),
+                    apply: right_shift,
                     prio: 2,
                     is_commutative: false,
                 },
@@ -549,7 +530,7 @@ where
             Operator::make_bin(
                 "<<",
                 BinOp {
-                    apply: |a, b| left_shift(a, b),
+                    apply: left_shift,
                     prio: 2,
                     is_commutative: false,
                 },
@@ -557,7 +538,7 @@ where
             Operator::make_bin(
                 "&&",
                 BinOp {
-                    apply: |a, b| and(a, b),
+                    apply: and,
                     prio: 2,
                     is_commutative: true,
                 },
@@ -565,7 +546,7 @@ where
             Operator::make_bin(
                 "||",
                 BinOp {
-                    apply: |a, b| or(a, b),
+                    apply: or,
                     prio: 2,
                     is_commutative: true,
                 },
@@ -633,7 +614,7 @@ where
                         }
                     },
                     prio: 0,
-                    is_commutative: true,
+                    is_commutative: false,
                 },
             ),
             Operator::make_bin(
@@ -644,42 +625,48 @@ where
                         _ => res_of_if,
                     },
                     prio: 0,
-                    is_commutative: true,
+                    is_commutative: false,
                 },
             ),
-            Operator::make_unary("signum", |a| signum(a)),
-            Operator::make_unary("abs", |a| abs(a)),
-            Operator::make_unary("sin", |a| sin(a)),
-            Operator::make_unary("cos", |a| cos(a)),
-            Operator::make_unary("tan", |a| tan(a)),
-            Operator::make_unary("asin", |a| asin(a)),
-            Operator::make_unary("acos", |a| acos(a)),
-            Operator::make_unary("atan", |a| atan(a)),
-            Operator::make_unary("sinh", |a| sinh(a)),
-            Operator::make_unary("cosh", |a| cosh(a)),
-            Operator::make_unary("tanh", |a| tanh(a)),
-            Operator::make_unary("floor", |a| floor(a)),
-            Operator::make_unary("ceil", |a| ceil(a)),
-            Operator::make_unary("trunc", |a| trunc(a)),
-            Operator::make_unary("fract", |a| fract(a)),
-            Operator::make_unary("exp", |a| exp(a)),
-            Operator::make_unary("sqrt", |a| sqrt(a)),
-            Operator::make_unary("cbrt", |a| cbrt(a)),
-            Operator::make_unary("round", |a| round(a)),
-            Operator::make_unary("log", |a| ln(a)),
-            Operator::make_unary("log2", |a| log2(a)),
-            Operator::make_unary("swap_bytes", |a| swap_bytes(a)),
-            Operator::make_unary("to_le", |a| to_le(a)),
-            Operator::make_unary("to_be", |a| to_be(a)),
-            Operator::make_unary("fact", |a| fact(a)),
-            Operator::make_unary("to_int", |a| cast_to_int(a)),
-            Operator::make_unary("to_float", |a| cast_to_float(a)),
+            Operator::make_unary("signum", signum),
+            Operator::make_unary("abs", abs),
+            Operator::make_unary("sin", sin),
+            Operator::make_unary("cos", cos),
+            Operator::make_unary("tan", tan),
+            Operator::make_unary("asin", asin),
+            Operator::make_unary("acos", acos),
+            Operator::make_unary("atan", atan),
+            Operator::make_unary("sinh", sinh),
+            Operator::make_unary("cosh", cosh),
+            Operator::make_unary("tanh", tanh),
+            Operator::make_unary("floor", floor),
+            Operator::make_unary("ceil", ceil),
+            Operator::make_unary("trunc", trunc),
+            Operator::make_unary("fract", fract),
+            Operator::make_unary("exp", exp),
+            Operator::make_unary("sqrt", sqrt),
+            Operator::make_unary("cbrt", cbrt),
+            Operator::make_unary("round", round),
+            Operator::make_unary("log", ln),
+            Operator::make_unary("log2", log2),
+            Operator::make_unary("swap_bytes", swap_bytes),
+            Operator::make_unary("to_le", to_le),
+            Operator::make_unary("to_be", to_be),
+            Operator::make_unary("fact", fact),
+            Operator::make_unary("to_int", cast_to_int),
+            Operator::make_unary("to_float", cast_to_float),
             Operator::make_constant("PI", Val::Float(F::from(std::f64::consts::PI).unwrap())),
             Operator::make_constant("π", Val::Float(F::from(std::f64::consts::PI).unwrap())),
             Operator::make_constant("E", Val::Float(F::from(std::f64::consts::E).unwrap())),
         ]
     }
 }
+const PATTERN: &str = r"^([0-9]+(\.[0-9]+)?|true|false|\[\s*(\-?.?[0-9]+(\.[0-9]+)?|true|false)(\s*,\s*-?\.?[0-9]+(\.[0-9]+)?|true|false)*\s*\])";
+literal_matcher_from_pattern!(ValMatcher, PATTERN);
+
+/// *`feature = "value"`* - Alias for [`FlatEx`](FlatEx) with [`Val`](Val) as data type and [`ValOpsFactory`](ValOpsFactory)
+/// as operator factory.
+pub type FlatExVal<I, F> = FlatEx<Val<I, F>, ValOpsFactory<I, F>, ValMatcher>;
 
 /// *`feature = "value"`* - Parses a string into an expression of type
 /// [`FlatExVal`](FlatExVal) with datatype [`Val`](Val).
@@ -695,178 +682,13 @@ where
 /// #     Ok(())
 /// # }
 /// ```
-pub fn parse_val<I, F>(text: &str) -> ExResult<FlatEx<Val<I, F>, ValOpsFactory<I, F>>>
+pub fn parse_val<I, F>(text: &str) -> ExResult<FlatExVal<I, F>>
 where
     I: DataType + PrimInt + Signed,
     F: DataType + Float,
     <I as FromStr>::Err: Debug,
     <F as FromStr>::Err: Debug,
 {
-    FlatEx::<Val<I, F>, ValOpsFactory<I, F>>::from_regex(text, &RE_VAR_NAME_EXACT)
+    FlatEx::<Val<I, F>, ValOpsFactory<I, F>, ValMatcher>::from_str(text)
 }
 
-/// *`feature = "value"`* - Parses a string into an expression of type [`OwnedFlatExVal`](OwnedFlatExVal) with
-/// datatype [`Val`](Val).
-pub fn parse_val_owned<I, F>(text: &str) -> ExResult<OwnedFlatEx<Val<I, F>, ValOpsFactory<I, F>>>
-where
-    I: DataType + PrimInt + Signed,
-    F: DataType + Float,
-    <I as FromStr>::Err: Debug,
-    <F as FromStr>::Err: Debug,
-{
-    Ok(OwnedFlatEx::from_flatex(parse_val(text)?))
-}
-
-#[cfg(test)]
-mod tests {
-
-    use crate::{
-        format_exerr,
-        util::assert_float_eq_f64,
-        value::{Val, RE_VAR_NAME_EXACT},
-        ExError, ExResult, Express, FlatEx,
-    };
-
-    use super::ValOpsFactory;
-    #[test]
-    fn test_to() -> ExResult<()> {
-        assert_eq!(Val::<i32, f64>::Float(3.4).to_float()?, 3.4);
-        assert_eq!(Val::<i32, f64>::Int(123).to_int()?, 123);
-        assert!(Val::<i32, f64>::Bool(true).to_bool()?);
-        assert!(Val::<i32, f64>::Bool(false).to_int().is_err());
-        assert_eq!(Val::<i32, f64>::Float(3.4).to_float()?, 3.4);
-        assert_eq!(Val::<i32, f64>::Int(34).to_int()?, 34);
-        assert!(!Val::<i32, f64>::Bool(false).to_bool()?);
-        Ok(())
-    }
-
-    #[test]
-    fn test_no_vars() -> ExResult<()> {
-        fn test_int(s: &str, reference: i32) -> ExResult<()> {
-            let res = FlatEx::<Val, ValOpsFactory>::from_regex(s, &RE_VAR_NAME_EXACT)?
-                .eval(&[])?
-                .to_int();
-            match res {
-                Ok(i) => {
-                    assert_eq!(reference, i);
-                }
-                Err(e) => {
-                    println!("{:?}", e);
-                    assert!(false)
-                }
-            }
-            Ok(())
-        }
-        fn test_float(s: &str, reference: f64) -> ExResult<()> {
-            let expr = FlatEx::<Val, ValOpsFactory>::from_regex(s, &RE_VAR_NAME_EXACT)?;
-            assert_float_eq_f64(reference, expr.eval(&[])?.to_float()?);
-            Ok(())
-        }
-        fn test_bool(s: &str, reference: bool) -> ExResult<()> {
-            let expr = FlatEx::<Val, ValOpsFactory>::from_regex(s, &RE_VAR_NAME_EXACT)?;
-            assert_eq!(reference, expr.eval(&[])?.to_bool()?);
-            Ok(())
-        }
-        fn test_error(s: &str) -> ExResult<()> {
-            let expr = FlatEx::<Val, ValOpsFactory>::from_regex(s, &RE_VAR_NAME_EXACT);
-            match expr {
-                Ok(exp) => {
-                    let v = exp.eval(&[])?;
-                    match v {
-                        Val::Error(e) => {
-                            println!("found expected error {:?}", e);
-                            Ok(())
-                        }
-                        _ => Err(format_exerr!("'{}' should fail but didn't", s)),
-                    }
-                }
-                Err(e) => {
-                    println!("found expected error {:?}", e);
-                    Ok(())
-                }
-            }
-        }
-        fn test_none(s: &str) -> ExResult<()> {
-            let expr = FlatEx::<Val, ValOpsFactory>::from_regex(s, &RE_VAR_NAME_EXACT)?;
-            match expr.eval(&[])? {
-                Val::None => Ok(()),
-                _ => Err(format_exerr!("'{}' should return none but didn't", s)),
-            }
-        }
-        test_error("929<<92")?;
-        test_error("929<<32")?;
-        test_error("929>>32")?;
-        test_int("928<<31", 0)?;
-        test_int("929>>31", 0)?;
-        test_float("2.0^2", 4.0)?;
-        test_int("2^4", 16)?;
-        test_error("2^-4")?;
-        test_int("2+4", 6)?;
-        test_int("9+4", 13)?;
-        test_int("9+4^2", 25)?;
-        test_int("9/4", 2)?;
-        test_int("9%4", 1)?;
-        test_float("2.5+4.0^2", 18.5)?;
-        test_float("2.5*4.0^2", 2.5 * 4.0 * 4.0)?;
-        test_float("2.5-4.0^-2", 2.5 - 4.0f64.powi(-2))?;
-        test_float("9.0/4.0", 9.0 / 4.0)?;
-        test_float("sin(9.0)", 9.0f64.sin())?;
-        test_float("cos(91.0)", 91.0f64.cos())?;
-        test_float("tan(913.0)", 913.0f64.tan())?;
-        test_float("sin(-π)", 0.0)?;
-        test_float("round(π)", 3.0)?;
-        test_float("cos(π)", -1.0)?;
-        test_float("sin (1 if false else 2.0)", 2.0f64.sin())?;
-        test_float("cbrt(27.0)", 3.0)?;
-        test_int("1 if true else 2.0", 1)?;
-        test_float("(9.0 if true else 2.0)", 9.0)?;
-        test_int("1<<4-2", 4)?;
-        test_int("4>>2", 1)?;
-        test_int("signum(4>>1)", 1)?;
-        test_float("signum(-123.12)", -1.0)?;
-        test_float("abs(-123.12)", 123.12)?;
-        test_int("fact(4)", 2 * 3 * 4)?;
-        test_int("fact(0)", 1)?;
-        test_error("fact(-1)")?;
-        test_bool("1>2", false)?;
-        test_bool("1<2", true)?;
-        test_bool("1.4>=1.4", true)?;
-        test_bool("true==true", true)?;
-        test_bool("false==true", false)?;
-        test_bool("1.5 != 1.5 + 2.0", true)?;
-        test_float("1 + 1.0", 2.0)?;
-        test_bool("1.0 == 1", true)?;
-        test_bool("1 == 1", true)?;
-        test_bool("2 == true", false)?;
-        test_bool("1.5 < 1", false)?;
-        test_bool("true == true", true)?;
-        test_bool("false != true", true)?;
-        test_bool("false != false", false)?;
-        test_bool("1 > 0.5", true)?;
-        test_error("to_float(10000000000000)")?;
-        test_bool("true == 1", false)?;
-        test_bool("true else 2", true)?;
-        test_int("1 else 2", 1)?;
-        test_error("if true else 2")?;
-        test_none("2 if false")?;
-        test_int("to_int(1)", 1)?;
-        test_int("to_int(3.5)", 3)?;
-        test_float("to_float(2)", 2.0)?;
-        test_float("to_float(3.5)", 3.5)?;
-        test_float("to_float(true)", 1.0)?;
-        test_float("to_float(false)", 0.0)?;
-        test_int("to_int(true)", 1)?;
-        test_int("to_int(false)", 0)?;
-        test_error("to_int(fact(-1))")?;
-        test_error("to_float(5 if false)")?;
-        test_error("0/0")?;
-        test_bool("(5 if false) == (5 if false)", false)?;
-        test_error("2^40")?;
-        test_error("1000000000*1000000000")?;
-        test_error("1500000000+1500000000")?;
-        test_error("-1500000000-1500000000")?;
-        test_error("0%0")?;
-
-        Ok(())
-    }
-}
